<?php

namespace Drupal\override_cache_control_headers\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;
use Drupal\override_cache_control_headers\Utility;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form that configures forms module settings.
 */
class OverrideCacheControlHeadersSettingsForm extends ConfigFormBase {

  /**
   * Config settings key.
   */
  const SETTINGS = 'override_cache_control_headers.settings';

  /**
   * Utility object.
   *
   * @var \Drupal\override_cache_control_headers\Utility
   */
  protected $utility;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Initiate utility and messenger service.
   *
   * @param Drupal\override_cache_control_headers\Utility $utility
   *   Cache control header utility class.
   * @param Drupal\Core\Messenger\MessengerInterface $messenger
   *   Messanger service Interface.
   */
  public function __construct(Utility $utility, MessengerInterface $messenger) {
    $this->utility = $utility;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('override_cache_control_headers_subscriber.utility'),
      $container->get('messenger'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'override_cache_control_headers_configuration';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $help_url = Url::fromUserInput('/admin/help/override_cache_control_headers');
    $help_link = Link::fromTextAndUrl($this->t('click here'), $help_url);
    $config = $this->config(static::SETTINGS);
    $form['urls_header'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Please enter the URL and headers separated by "|" symbol.'),
      '#default_value' => $config->get('urls_header') ?? '',
      '#description' => $this->t(
        'Example: 
        <b>@example</b>
        <br>Enter one entry per line.
        <br>For a list of possible values, @link.',
        [
          '@example' => '/sitemap.xml|must-revalidate, no-cache, private',
          '@link' => $help_link->toString(),
        ]
      ),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $urls_header = $form_state->getValue('urls_header');
    if (!empty($urls_header)) {
      $urls_headers = array_map('trim', explode(PHP_EOL, $urls_header));
      if ((is_array($urls_headers)) && (!empty($urls_headers))) {
        $response = $this->utility->validateCacheControlStrings($urls_headers);
        if (!empty($response)) {
          $form_state->setErrorByName('urls_header');
          foreach ($response as $error) {
            $this->messenger->addError($error);
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->configFactory->getEditable(static::SETTINGS)
      ->set('urls_header', $form_state->getValue('urls_header'))
      ->save();
    parent::submitForm($form, $form_state);
  }

}
